#include <LiquidCrystal.h>  //LCD kütüphanesi ekleniyor
#include <TimerOne.h> //Timer kütüphanesi ekleniyor. (internetten indirdiğimiz)

LiquidCrystal ekran(13, 12, 11, 10, 9, 8);  //Ekran nesnesi oluşturuluyor.
                                            //RS=13,EN=12,D4=11,D5=10,D6=9,D7=9
const int STEP1 = 7;                        //Adım motorunun bacaklarının bağlı olduğu çıkışlar
const int STEP2 = 6;
const int STEP3 = 5;
const int STEP4 = 4;

#define BTN_BASILI LOW                //Butona basıldığında entegre girişlerine dijital sıfır değeri gelecek
const int BTN_TAM_YARIM = 3;          //Butonların bağlı olduğu bacaklar
const int BTN_ILERI_GERI = 2;

#define STEP_TAM true               //Adım motorunun iki durumu için de tanımlamalar
#define STEP_YARIM false
bool adimDurumu = STEP_TAM;         //Adım motorunun çalışma modunu hafızasında tutan durum değişkeni

#define ILERI_YON true              //Adım motorunun yön bilgileri
#define GERI_YON false
bool adimYon = ILERI_YON;           //Adım motorunun yön bilgisini tutan durum değişkeni

long zamanAsimi;                   // Buton basıldığındaki zaman aşımı için gerekli değişken
int sayac = 0;                      // Adım motorunun tetikleme sıralamaları için kullanılan değişken

void setup() {

  pinMode(STEP1, OUTPUT);       // Adım motorunun bacakları çıkış haline getiriliyor.
  pinMode(STEP2, OUTPUT);
  pinMode(STEP3, OUTPUT);
  pinMode(STEP4, OUTPUT);

  pinMode(BTN_TAM_YARIM, INPUT_PULLUP); //Buton bacakları giriş olarak tanımlanıyor. Dahili pullup dirençleri açık
  pinMode(BTN_ILERI_GERI, INPUT_PULLUP);

  ekran.begin(20, 4);                   //LCD ekran ayarları yapılıyor.
  
  Timer1.initialize(100000);                  //Timer kesmesi hazırlanıyor.
  Timer1.attachInterrupt( timerKesmesi );     //Kesme fonksiyonu ekleniyor.

}
int analogHiz = 0;      //AN0 bacağından okunan hız bilgisini tutan değişken
float hizD_DK = 0;      //Devir/Dakika cinsinden hız bilgisini tutan değişken
int tur_Pals;           // Adım motorunun moduna göre bir tur için gerekli pals sayısını tutan değişken
char txtEkran[10];      // Float sayıyı LCD ekrana yazdırmak için gerekli text hafızası
void loop() {
  stepSur(sayac);         //Adım motoru için gerekli çıkışları üreten alt fonksiyon
  butonlariKontrolEt();   //Butonları okuyan alt fonksiyon
  hizOku();               //Analog bacaktan hız bilgisini okuyan alt fonksiyon
  ekranaYaz();            //Ekrana gerekli bilgileri yazan alt fonksiyon
}
void hizOku() {
  analogHiz = map(analogRead(A0), 0, 1023, 100, 1); //Analog bacaktan hız bilgisi okunuyor. (100ms - 1ms)
  Timer1.initialize(analogHiz * 1000);              //Okunan zaman mikrosaniyeye çevirilerek timer kesme süresi güncelleniyor.
}
void ekranaYaz() {
  ekran.setCursor(0, 0);                    //Kursor ekranda 0,0 konumuna getiriliyor.
  ekran.print("HIZ:");                      //Ekrana HIZ: yazdırılıyor.
  if (adimDurumu == STEP_TAM)tur_Pals = 300;  //Hız hesabı için adım motorunun sürüş moduna göre bir tur atması için gereken pals sayısı değişkene yükleniyor
  else tur_Pals = 600;
  hizD_DK = 60e3 / (tur_Pals * (float)analogHiz); //Devir/Dakika cinsinden hız hesaplanıyor.
  dtostrf(hizD_DK, 4, 1, txtEkran); //Hesaplanan hız text formatına çeviriliyor.
  ekran.print(txtEkran);  //LCD ekrana yazdırılıyor.
  ekran.print(" d/dk ");   //hız bilgisinin sonuna d/dk yazılıyor.
  ekran.setCursor(0, 1);  //Kursor 2. satıra indiriliyor.
  ekran.print("YON:");    //Ekrana YON: yazdırılıyor.
  if (adimYon == ILERI_YON)ekran.print("ILERI");  //Seçilen yöne göre ekrana GERI yada ILERI yazdırılıyor.
  else ekran.print("GERI ");
  ekran.setCursor(0, 2);  //Kursor 3. satıra indiriliyor.
  ekran.print("ADIM:");   //Ekrana ADIM: yazdırılıyor.
  if (adimDurumu == STEP_TAM)ekran.print("TAM  "); //Seçilen yöne göre ekrana TAM yada YARIM yazdırılıyor.
  else ekran.print("YARIM");
}
void butonlariKontrolEt() {
  if (digitalRead(BTN_TAM_YARIM) == BTN_BASILI) { //butona basıldıysa adimDurumu değiştir
    if (adimDurumu == STEP_TAM) {
      adimDurumu = STEP_YARIM;
    } else {
      adimDurumu = STEP_TAM;
    }

    zamanAsimi = millis();
    while (digitalRead(BTN_TAM_YARIM) == BTN_BASILI) { // Eğer butona basılma süresi 1 saniyeyi geçerse çık
      if (millis() - zamanAsimi > 1000)break;
    }
  }

  if (digitalRead(BTN_ILERI_GERI) == BTN_BASILI) { //butona basıldıysa adimYön bilgisini değiştir
    if (adimYon == ILERI_YON) {
      adimYon = GERI_YON;
    } else {
      adimYon = ILERI_YON;
    }

    zamanAsimi = millis();
    while (digitalRead(BTN_ILERI_GERI) == BTN_BASILI) { // Eğer butona basılma süresi 1 saniyeyi geçerse çık
      if (millis() - zamanAsimi > 1000)break;
    }
  }
}
void timerKesmesi() {               //Derste anlatılan sayac arttırma işlemi süreye bağlı olarak çalışması için timer fonksiyonuna yazıldı.
  if (adimDurumu == STEP_TAM) { 
    if (adimYon == ILERI_YON) {
      if (sayac > 0)sayac -= 2;
      else sayac = 7;

    } else {
      if (sayac < 7)sayac += 2;
      else sayac = 0;
    }
  } else {
    if (adimYon == ILERI_YON) {
      if (sayac > 0)sayac --;
      else sayac = 7;
    } else {
      if (sayac < 7)sayac ++;
      else sayac = 0;
    }
  }
}
void stepSur(int durum) { //Adım motorunun tam veya yarım adım için gerekli çıkış bilgileri
  switch (durum) {
    case 0:
      digitalWrite(STEP1, HIGH);
      digitalWrite(STEP2, LOW);
      digitalWrite(STEP3, LOW);
      digitalWrite(STEP4, LOW);
      break;
    case 1:
      digitalWrite(STEP1, HIGH);
      digitalWrite(STEP2, HIGH);
      digitalWrite(STEP3, LOW);
      digitalWrite(STEP4, LOW);
      break;
    case 2:
      digitalWrite(STEP1, LOW);
      digitalWrite(STEP2, HIGH);
      digitalWrite(STEP3, LOW);
      digitalWrite(STEP4, LOW);
      break;
    case 3:
      digitalWrite(STEP1, LOW);
      digitalWrite(STEP2, HIGH);
      digitalWrite(STEP3, HIGH);
      digitalWrite(STEP4, LOW);
      break;
    case 4:
      digitalWrite(STEP1, LOW);
      digitalWrite(STEP2, LOW);
      digitalWrite(STEP3, HIGH);
      digitalWrite(STEP4, LOW);
      break;
    case 5:
      digitalWrite(STEP1, LOW);
      digitalWrite(STEP2, LOW);
      digitalWrite(STEP3, HIGH);
      digitalWrite(STEP4, HIGH);
      break;
    case 6:
      digitalWrite(STEP1, LOW);
      digitalWrite(STEP2, LOW);
      digitalWrite(STEP3, LOW);
      digitalWrite(STEP4, HIGH);
      break;
    case 7:
      digitalWrite(STEP1, HIGH);
      digitalWrite(STEP2, LOW);
      digitalWrite(STEP3, LOW);
      digitalWrite(STEP4, HIGH);
      break;
  }
}

